﻿#region LICENSE
// ---------------------------------- LICENSE ---------------------------------- //
//
//    Fling OS - The educational operating system
//    Copyright (C) 2015 Edward Nutting
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 2 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//  Project owner: 
//		Email: edwardnutting@outlook.com
//		For paper mail address, please contact via email for details.
//
// ------------------------------------------------------------------------------ //
#endregion
    
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Drivers.Compiler.Architectures.x86.ASMOps
{
    public class StringLiteral : ASM.ASMOps.ASMStringLiteral
    {
        public StringLiteral(string id, string stringTypeId, byte[] lengthBytes, char[] characters)
            : base(id, stringTypeId, lengthBytes, characters)
        {
        }

        public override string Convert(ASM.ASMBlock theBlock)
        {
            StringBuilder LiteralASM = new StringBuilder();
            //This is UTF-16 (Unicode)/ASCII text
            LiteralASM.AppendLine(string.Format("GLOBAL {0}:data", Id));
            LiteralASM.AppendLine(string.Format("{0}:", Id));
            //Put in type info as FOS_System.String type
            LiteralASM.AppendLine(string.Format("dd {0}", StringTypeId));
            //Put in string length bytes
            LiteralASM.Append("db ");
            for (int i = 0; i < 3; i++)
            {
                LiteralASM.Append(LengthBytes[i]);
                LiteralASM.Append(", ");
            }
            LiteralASM.Append(LengthBytes[3]);

            if (Characters.Length > 0)
            {
                //Put in string characters (as words)
                LiteralASM.Append("\ndw ");
                for (int i = 0; i < (Characters.Length - 1); i++)
                {
                    LiteralASM.Append((uint)Characters[i]);
                    LiteralASM.Append(", ");
                }
                LiteralASM.Append((uint)Characters.Last());
            }

            LiteralASM.AppendLine();

            return LiteralASM.ToString();
        }
    }
}
